/*#######################################################################################
ARM Linux MMC Boot Loader

Copyright (C) 2004 Ulrich Radig

Bei Fragen und Verbesserungen wendet euch per EMail an

mail@ulrichradig.de

oder im Forum meiner Web Page : www.ulrichradig.de


Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
GNU General Public License, wie von der Free Software Foundation verffentlicht, 
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
(nach Ihrer Option) jeder spteren Version. 

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
Programm erhalten haben. 
Falls nicht, schreiben Sie an die Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#######################################################################################*/

#include "main.h"

/*#######################################################################################
Config System
#######################################################################################*/
void SystemInit(void)
{

	// --- enable and connect the PLL (Phase Locked Loop) ---
	// a. set multiplier and divider
	SCB_PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	// b. enable PLL
	SCB_PLLCON = (1<<PLLE);
	// c. feed sequence
	SCB_PLLFEED = PLL_FEED1;
	SCB_PLLFEED = PLL_FEED2;
	// d. wait for PLL lock (PLOCK bit is set if locked)
	while (!(SCB_PLLSTAT & (1<<PLOCK)));
	// e. connect (and enable) PLL
	SCB_PLLCON = (1<<PLLE) | (1<<PLLC);
	// f. feed sequence
	SCB_PLLFEED = PLL_FEED1;
	SCB_PLLFEED = PLL_FEED2;
	
	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAM_MAMCR = 0;	
	
	SCB_EXTPOLAR= 0x04;
	SCB_EXTINT	= 0x04;
	
	
	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAM_MAMTIM = MAM_FETCH;
	// c. enable MAM 
	MAM_MAMCR = MAM_MODE;
	
	// --- set VPB speed ---
	SCB_VPBDIV = VPBDIV_VAL;
	
	// --- map INT-vector ---
	 SCB_MEMMAP = MEMMAP_USER_FLASH_MODE;
}

void swiirqp (void) 
{	
	printf("\nint\n");
}

/*#######################################################################################
Main Programm
#######################################################################################*/
int main(void)
{

	void (*kernelstart)(void);
  
	SystemInit();
 
	uart0Init(B9600, UART_8N1, UART_FIFO_OFF); // setup the UART
	
	PCB_PINSEL2 = 0x0F814924; //Init Extendet Memory
	BCFG1 = 0x20000400;

	
	printf("\n\n\r");
	printf("*******************************************\n\r");
	printf("*                                         *\n\r");
	printf("*    ARM Linux MMC Boot Loader            *\n\r");
	printf("*                                         *\n\r");
	printf("*    Copyright (C) 2005 by Ulrich Radig   *\n\r");
	printf("*                                         *\n\r");
	printf("*******************************************\n\n\r");
	while (mmc_init() != 0) 	//setup the MMC Card
			{
			   printf("**MMC/SD - Card not found! **\r"); 
			}

	//Speichern der Cluster Size und Cluster Offset (Anfang Datenbereich)
	Cluster_Data_Store();
	//Initialisierung der MMC/SD-Karte ENDE!

	load_file (FILE1,(int*)KERNEL_START);
	load_file (FILE2,(int*)BOOTIMG_ADDR);
	
	printf("\n\n\rLinux wird gestartet!\n\n\r"); 
	
	PCB_PINSEL0 = 0x80000005;
	PCB_PINSEL2 = 0x0F814924;
	
	BCFG0 = 0x2000FFEF;
	BCFG1 = 0x20000400;
	BCFG2 = 0x0000FFEF;
	BCFG3 = 0x0000FFEF;

	VICDefVectAddr = 0x81000018;
	
	kernelstart = (void (*)(void))KERNEL_START;//KERNEL_START;		
	(*kernelstart)();  
	
	printf("Never reached this!!");
	
	while(1)
		{
		} 
return(0);
}


